# -*- coding: utf-8 -*-

from keras.models import Sequential
from tensorflow.keras import layers
from tensorflow.keras.utils import to_categorical
from sklearn.model_selection import train_test_split
from pathlib import Path
import numpy as np
import cv2
import csv


# parameters to be changed
num_classes = 4                            # number of classification class
input_dir = Path.cwd() / r'PATH'           # image path
files = list(input_dir.rglob("*.png"))     # image type
file_y = 'PATH'                            # target path


X_img,y_img = [],[]

# read images into X_img like grayscale
for path in files:
    img = cv2.imread(str(path),0)
    X_img.append(img)
X_img = np.array(X_img)

# image size
img_height = X_img.shape[1]
img_width = X_img.shape[2]


# read target values from file into y
with open(file_y, 'r') as file:
    reader = csv.reader(file,delimiter = '\n')
    for row in reader:
        y_img.append(int(row[0]))

y_one_hot = to_categorical(y_img,num_classes)      

# train and test split
X_img_train, X_img_test, y_img_train, y_img_test = train_test_split(X_img,y_one_hot,test_size=0.25)


# CNN architecture
model = Sequential([
  layers.Rescaling(1./255, input_shape=(img_height, img_width, 1)),
  layers.Conv2D(64, (3,3), padding='same', activation='relu'),
  layers.MaxPooling2D(pool_size=(2,2)),
  layers.Conv2D(64, (3,3), padding='same', activation='relu'),
  layers.MaxPooling2D(pool_size=(2,2)),
  layers.Conv2D(64, (3,3), padding='same', activation='relu'),
  layers.MaxPooling2D(pool_size=(2,2)),
  layers.Flatten(),
  layers.Dense(64),
  layers.Dense(num_classes, activation='softmax')
])

# model compile
model.compile(loss='categorical_crossentropy',optimizer='adam',metrics=['accuracy'])

# model fit
model.fit(X_img_train,y_img_train, epochs=10, validation_split=0.2)

# evaluation
print('Evaluation: ')
model.evaluate(X_img_test,y_img_test)[1]

# prediction
predictions_percent = model.predict(X_img_test)

predictions = []
for i in predictions_percent:
    predictions.append(i.argmax())

# save model 
# model.save('name')    # uncomment to save the model

